unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, simpleipc, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, Grids, ValEdit, CheckLst, PairSplitter, FileUtil, strutils;

type

  { TForm1 }

  TForm1 = class(TForm)
    Log: TMemo;
    ShowSuspendedButton: TButton;
    ShowListButton: TButton;
    IPServer: TSimpleIPCServer;
    SuspendButton: TToggleBox;
    SuspendTimer: TTimer;
    UnlockFileButton: TButton;
    UnlockFolderButton: TButton;
    ListBox: TListBox;
    Shape1: TShape;
    SubDirsCheckBox: TCheckBox;
    SelectFolderDialog: TSelectDirectoryDialog;
    SelectedEdit: TEdit;
    LockFolderButton: TButton;
    LocateButton: TButton;
    TrayIcon: TTrayIcon;
    procedure Button1Click(Sender: TObject);
    procedure FormWindowStateChange(Sender: TObject);
    procedure IPServerMessage(Sender: TObject);
    procedure ShowSuspendedButtonClick(Sender: TObject);
    procedure ListBoxClick(Sender: TObject);
    procedure ShowListButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure LocateButtonClick(Sender: TObject);
    procedure LockFolderButtonClick(Sender: TObject);
    procedure RefreshList;
    procedure SuspendButtonChange(Sender: TObject);
    procedure SuspendTimerTimer(Sender: TObject);
    procedure TrayIconClick(Sender: TObject);
    procedure UnlockFileButtonClick(Sender: TObject);
    procedure UnlockFolderButtonClick(Sender: TObject);
    procedure SuspendLock(Filename: string);
    procedure SuspendEnd(Filename: string);
  private

  public              //  ↓↓↓↓↓↓↓  Maximum lockable file count Write over if your device can hold it
    DenyStreams: array[0..999999] of TFileStream;
    FileList: TStringList;
    UnlockList: TStringList;
    Suspended: TStringList;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

const
  FilePassword: string = '45gg34grdh5';

var
  Password: string;
  IncomingSuspendMessages: string;

procedure TForm1.SuspendLock(Filename: string);
var
  i, i2: integer;
begin
  if filename <> '' then
    for i := 0 to high(DenyStreams) do
      if Assigned(DenyStreams[i]) then
        if Filename = DenyStreams[i].FileName then
          for i2 := 0 to FileList.Count - 1 do
            if Filename = FileList.Strings[i2] then
            begin
              FreeAndNil(DenyStreams[i]);
              Suspended.Add(filename);
              FileList.Delete(i2);
              exit;
            end;
end;

procedure TForm1.SuspendEnd(Filename: string);
var
  i, i2: integer;
begin

  if Filename <> '' then
    for i := 0 to Suspended.Count - 1 do
      if Filename = Suspended.Strings[i] then
      begin
        Suspended.Delete(i);
        FileList.Add(Filename);
        for i2 := 0 to High(DenyStreams) do
          if not Assigned(DenyStreams[i2]) then
          begin
            DenyStreams[i2] :=
              TFileStream.Create(Filename, fmOpenRead or fmShareDenyRead or
              fmShareDenyWrite);
            exit;
          end;
      end;

end;

procedure TForm1.RefreshList;
var
  i: integer;
begin
  ListBox.Clear;
  if Assigned(FileList) then
    for i := 0 to FileList.Count - 1 do
      if FileList.Strings[i] <> '' then
        ListBox.AddItem(FileList.Strings[i], self);
  if Assigned(Suspended) then
    for i := 0 to Suspended.Count - 1 do
      if Suspended.Strings[i] <> '' then
        ListBox.AddItem(Suspended.Strings[i], Self);
end;

procedure TForm1.SuspendButtonChange(Sender: TObject);
begin
  if ListBox.Items[ListBox.ItemIndex] <> '' then
  begin
    if SuspendButton.Checked = True then
      SuspendLock(ListBox.Items[ListBox.ItemIndex]);
    if SuspendButton.Checked = False then
      SuspendEnd(ListBox.Items[ListBox.ItemIndex]);
  end;
end;

procedure TForm1.SuspendTimerTimer(Sender: TObject);
begin
  if IPServer.Active then
    if IPServer.PeekMessage(100, True) then
    begin
      if ExtractDelimited(1, IncomingSuspendMessages, ['&']) = '{' then
        //REMOTE SUSPEND STARTS
        if ExtractDelimited(2, IncomingSuspendMessages, ['&']) = FilePassword then
        begin
          SuspendLock(ExtractDelimited(3, IncomingSuspendMessages, ['&']));
        end;

      if IPServer.StringMessage[1] = '}' then //RENMOTE SUSPEND ENDS
        if ExtractDelimited(2, IncomingSuspendMessages, ['&']) = FilePassword then
        begin
          SuspendEnd(ExtractDelimited(3, IncomingSuspendMessages, ['&']));
        end;
    end;
end;

procedure TForm1.TrayIconClick(Sender: TObject);
begin
  WindowState := wsNormal;
  TrayIcon.Visible := False;
  Form1.Show;
end;

procedure TForm1.UnlockFileButtonClick(Sender: TObject);
var
  i: integer;
begin
  if ListBox.Items[ListBox.ItemIndex] <> '' then
    for i := 0 to high(DenyStreams) do
      if assigned(DenyStreams[i]) then
        if ListBox.Items[ListBox.ItemIndex] = DenyStreams[i].FileName then
        begin
          FreeAndNil(DenyStreams[i]);
          FileList.Delete(i);
        end;
  RefreshList;
end;

procedure TForm1.UnlockFolderButtonClick(Sender: TObject);
var
  i, i2, i3: integer;
  Waitings: integer;
label
  Going;
begin
  try
    SetCurrentDir(SelectedEdit.Text);
    UnlockList := TStringList.Create;
    UnlockList.Duplicates := dupIgnore;
    UnlockList.AddStrings(FindAllFiles(GetCurrentDir, '*', SubDirsCheckBox.Checked,
      faDirectory));

    Going:
      for i := 0 to UnlockList.Count - 1 do
      begin
        if UnlockList.Strings[i] <> '' then
          for i2 := 0 to high(DenyStreams) - 1 do
            if Assigned(DenyStreams[i2]) then
            begin
              if UnlockList.Strings[i] = DenyStreams[i2].FileName then begin
                for i3 := 0 to FileList.Count - 1 do
                  if FileList.Strings[i3] <> '' then
                    if UnlockList.Strings[i] = FileList.Strings[i3] then
                    begin
                      UnlockList.Delete(i);
                      FileList.Delete(i3);
                      FreeAndNil(DenyStreams[i2]);
                      goto Going;
                    end;
              end else
                if i3=UnlockList.Count -1 then UnlockList.Delete(i3);
            end;
      end;

  finally
    RefreshList;
  end;
  RefreshList;
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  if PasswordBox('Authentication', 'Type the password:') <> Password then
    CloseAction := caNone
  else
    FileList.Free;
end;

procedure TForm1.ShowListButtonClick(Sender: TObject);
begin
  if Assigned(FileList) then
    ShowMessage(FileList.Text);
  RefreshList;
end;

procedure TForm1.ListBoxClick(Sender: TObject);
var
  i, i2: integer;
begin
  for i := 0 to Suspended.Count - 1 do
    if Suspended.find(ListBox.Items[ListBox.ItemIndex], i2) = True then
      SuspendButton.Checked := True
    else
      SuspendButton.Checked := False;
end;

procedure TForm1.ShowSuspendedButtonClick(Sender: TObject);
begin
  ShowMessage(Suspended.Text);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin

end;

procedure TForm1.FormWindowStateChange(Sender: TObject);
begin
  if WindowState = wsMaximized then
    WindowState := wsNormal;
  if WindowState = wsFullScreen then
    WindowState := wsNormal;
  if WindowState = wsMinimized then
  begin
    TrayIcon.Visible := True;
    Form1.Hide;
  end;
end;

procedure TForm1.IPServerMessage(Sender: TObject);
begin
  IncomingSuspendMessages := IPServer.StringMessage;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  TrayIcon.Icon := Form1.Icon;
  Application.Icon := form1.Icon;

  Suspended := TStringList.Create;
  Suspended.Duplicates := dupIgnore;
  Suspended.Sorted := True;

  FileList := TStringList.Create;
  FileList.Duplicates := dupIgnore;
  FileList.Sorted := True;

  Password := PasswordBox('Setting', 'Type the shutdown password for this runtime:');
end;

procedure TForm1.LocateButtonClick(Sender: TObject);
begin
  if SelectFolderDialog.Execute then
    SelectedEdit.Text := SelectFolderDialog.FileName;
end;

procedure TForm1.LockFolderButtonClick(Sender: TObject);
var
  i, i2: integer;
label
  x;
begin
  SetCurrentDir(SelectedEdit.Text);
  FileList.AddStrings(FindAllFiles(GetCurrentDir, '*', SubDirsCheckBox.Checked,
    faDirectory));
  ShowMessage('Theese are detected:' + #13#10 + FileList.Text);
  for i := 0 to High(DenyStreams) do
    if Assigned(DenyStreams[i]) then
      FreeAndNil(DenyStreams[i]);
  x:
    for i := 0 to FileList.Count - 1 do
    begin
      if Assigned(Suspended) then
        for i2 := 0 to Suspended.Count - 1 do
          if FileList.Strings[i] = Suspended.Strings[i2] then
          begin
            FileList.Delete(i);
            goto x;
          end;
      if FileList.Strings[i] <> '' then
        if not Assigned(DenyStreams[i]) then
          DenyStreams[i] := TFileStream.Create(FileList.Strings[i],
          fmOpenRead or fmShareDenyRead or fmShareDenyWrite);
    end;

  ShowMessage('Theese will be locked:' + #13#10 + FileList.Text);
  RefreshList;

  if IPServer.Active = False then
    IPServer.StartServer;

end;

end.
